package com.ashokit.ticketbooking.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ashokit.ticketbooking.model.Ticket;
import com.ashokit.ticketbooking.service.TicketService;

@RestController
@RequestMapping(value="/tickets")
//http://localhost:8080/tickets==> TicketController obj=ioc.getTicketController();
public class TicketController {
	
	
	/*
	 * 
	 * 
	 * @RestController ==> Exposing Ticket  Booking Functionality to outside world
	 * 
	 * Through REST Services ==> JAX RS ==> Java API for RestServices
	 * 
	 * REST Services ==> To achieve hetero genious communication we can use rest services
	 *               ==> Common Language ==> JSON ==> Java Script Object Notation
	 *               
	 *  VERBS ==> @GET ==>Retieve the Data + @RequestMapping ==>@GetMapping
	 *  
	 *  	  ==> @POST ==> Creates the Data + @RequestMapping ==> @PostMapping
	 *  
	 *         ==> @PUT ==> Updates the Data + @RequestMapping ==> @PutMapping
	 *         
	 *         ==> @DELETE ==> Delete the Data + @RequestMapping ==> @DeleteMapping
	 * 
	 * 
	 *  Spring Boot automatically takes care of Marshalling and UnMarshalling with JackSon Library
	 *  
	 *  Marshalling ==> Java to JSON Conversion
	 *  
	 *  UnMarshalling ==> JSON TO Java Conversion
	 */

	
	@Autowired
	private TicketService ticketService;
	
	//Retrieve All Tickets
	//All the below java responses will be converted into JSON by JackSon Library
	
	@GetMapping(value="/all")
	//http://localhost:8080/tickets/all==> obj.getAllTickets()
	public Iterable<Ticket> getAllTickets(){
		return ticketService.getAllTickets();
	}
	
	
	//Retriving Individual Ticket
	//localhost:8080/tickets/1 ==> 1st Ticket
	//localhost:8080/tickets/2 ==> 2nd Ticket
	//localhost:8080/tickets/100 ==> 100th Ticket
	//@PathVariable ==> This annotation will read the path varaiable in url and replaces with input value
	/*@GetMapping(value="/{ticketId}")
	public Ticket getTicket(@PathVariable("ticketId") Integer ticketId) {
		return ticketService.getTicket(ticketId);
	}*/
	
	
	@GetMapping(value="/{passengerName}")
	//localhost:8080/tickets/Ramu
	//localhost:8080/tickets/Sandhya
	public Ticket getTicketByPassenger(@PathVariable("passengerName")  String passengerName) {
		return ticketService.findByPassengerName(passengerName);
	}
	
	
	//Create Ticket API
	@PostMapping(value="/create")
	//http://localhost:8080/tickets/create ==>obj.createTicket()
	//Jackson libaray convert ticket.json to ticket.java
	//@RequestBody ==> Is going to verify the body section and scans the JSON object and converts into ticket object
	public Ticket createTicket(@RequestBody Ticket ticketObj) {
		return ticketService.CreateTicket(ticketObj);
	}
	
	//Update Ticket API
	@PutMapping(value="/{ticketId}/{newEmail}")
	//http://localhost:8080/tickets/1/vijay@yahoo.com
	public Ticket updateTicket(@PathVariable("ticketId") Integer ticketId,
							   @PathVariable("newEmail") String newEmail) {
		return ticketService.updateTicket(ticketId, newEmail);
	}
	
	//Delete Ticket
	
	@DeleteMapping(value="/{ticketId}")
	//http://localhost:8080/tickets/1 ==> Deletes the 1st Ticket
	//http://localhost:8080/tickets/10 ==> Deletes the 10th Ticket
	//http://localhost:8080/tickets/100 ==> Deletes the 100th Ticket
	public void deleteTicket(@PathVariable("ticketId") Integer ticketId) {
		ticketService.deleteTicket(ticketId);
	}
}
